unit B25D_delphi;

interface

uses
  Windows;

const
  DLLNAME = 'B25Decoder_d.dll';

type
  TCreateB25Decoder      = function :Pointer; cdecl;
  TB25Decoder_Initialize = function (B25D:Pointer; dwRound:DWORD):BOOL; cdecl;
  TB25Decoder_Release    = procedure(B25D:Pointer); cdecl;
  TB25Decoder_Decode     = function (B25D:Pointer; pSrcBuf:PByte; const dwSrcSize:DWORD; var ppDstBuf:PByte; var pdwDstSize:DWORD):BOOL; cdecl;
  TB25Decoder_Flush      = function (B25D:Pointer; var ppDstBuf:PByte; var pdwDstSize:DWORD):BOOL; cdecl;
  TB25Decoder_Reset      = function (B25D:Pointer):BOOL; cdecl;

  TB25Decoder = class(TObject)
  private
    FB25Dec      :Pointer;
    FDLL         :DWORD;
    FInitialized :Boolean;
    CreateB25Decoder      :TCreateB25Decoder;
    B25Decoder_Initialize :TB25Decoder_Initialize;
    B25Decoder_Release    :TB25Decoder_Release;
    B25Decoder_Decode     :TB25Decoder_Decode;
    B25Decoder_Flush      :TB25Decoder_Flush;
    B25Decoder_Reset      :TB25Decoder_Reset;
    function    GetDLLReady:Boolean;
  public
    constructor Create;
    destructor  Destroy; override;
    function    Initialize(dwRound:DWORD):BOOL;
    procedure   Release;
    function    Decode(pSrcBuf:PByte; const dwSrcSize:DWORD; var ppDstBuf:PByte; var pdwDstSize:DWORD):BOOL;
    function    Flush(var ppDstBuf:PByte; var pdwDstSize:DWORD):BOOL;
    function    Reset:BOOL;
    property    DLLReady:Boolean read GetDLLReady;
    property    Initialized:Boolean read FInitialized;
  end;

implementation

constructor TB25Decoder.Create;
var
  flg:Boolean;
  pProc: Pointer;
begin
  flg          := False;
  FDLL         := LoadLibrary(DLLNAME);
  FB25Dec      := nil;
  FInitialized := False;
  if (FDLL<>0) then
  begin
    pProc := GetProcAddress(FDLL, 'CreateB25Decoder');
    if (pProc=nil) then flg := True;
    CreateB25Decoder := pProc;
    //
    pProc := GetProcAddress(FDLL, 'B25Decoder_Initialize');
    if (pProc=nil) then flg := True;
    B25Decoder_Initialize := pProc;
    //
    pProc := GetProcAddress(FDLL, 'B25Decoder_Release');
    if (pProc=nil) then flg := True;
    B25Decoder_Release := pProc;
    //
    pProc := GetProcAddress(FDLL, 'B25Decoder_Decode');
    if (pProc=nil) then flg := True;
    B25Decoder_Decode := pProc;
    //
    pProc := GetProcAddress(FDLL, 'B25Decoder_Flush');
    if (pProc=nil) then flg := True;
    B25Decoder_Flush := pProc;
    //
    pProc := GetProcAddress(FDLL, 'B25Decoder_Reset');
    if (pProc=nil) then flg := True;
    B25Decoder_Reset := pProc;
    //
    if (flg) then
    begin
      CreateB25Decoder      := nil;
      B25Decoder_Initialize := nil;
      B25Decoder_Release    := nil;
      B25Decoder_Decode     := nil;
      B25Decoder_Flush      := nil;
      B25Decoder_Reset      := nil;
      FreeLibrary(FDLL);
      FDLL := 0;
    end
      else
    begin
      FB25Dec := CreateB25Decoder();
    end;
  end;
end;

destructor TB25Decoder.Destroy;
begin
  if (Assigned(FB25Dec) and Assigned(B25Decoder_Release)) then B25Decoder_Release(FB25Dec);
  if (FDLL<>0) then FreeLibrary(FDLL);
end;

function TB25Decoder.GetDLLReady:Boolean;
begin
  Result := (FB25Dec<>nil);
end;

function TB25Decoder.Initialize(dwRound:DWORD):BOOL;
begin
  Result := False;
  if Assigned(FB25Dec) then
  begin
    Result := B25Decoder_Initialize(FB25Dec,dwRound);
    if (Result) then FInitialized := True;
  end;
end;

procedure TB25Decoder.Release;
begin
  if Assigned(FB25Dec) then
  begin
    B25Decoder_Release(FB25Dec);
    FInitialized := False;
    FB25Dec := nil;
  end;
end;

function TB25Decoder.Decode(pSrcBuf:PByte; const dwSrcSize:DWORD; var ppDstBuf:PByte; var pdwDstSize:DWORD):BOOL;
begin
  Result := False;
  if Assigned(FB25Dec) then Result := B25Decoder_Decode(FB25Dec,pSrcBuf,dwSrcSize,ppDstBuf,pdwDstSize);
end;

function TB25Decoder.Flush(var ppDstBuf:PByte; var pdwDstSize:DWORD):BOOL;
begin
  Result := False;
  if Assigned(FB25Dec) then Result := B25Decoder_Flush(FB25Dec,ppDstBuf,pdwDstSize);
end;

function TB25Decoder.Reset:BOOL;
begin
  Result := False;
  if Assigned(FB25Dec) then
  begin
    Result := B25Decoder_Reset(FB25Dec);
    if (Result) then FInitialized := True;
  end;
end;


end.
