unit HDUS_BDA;

// ----------------------------------------------
//   HDUS_BDA
//   MobileHackerz : http://mobilehackerz.jp
// ----------------------------------------------
(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  This Program is free software; you can redistribute it and/or modify     *
 *  it under the terms of the GNU General Public License as published by     *
 *  the Free Software Foundation; either version 2, or (at your option)      *
 *  any later version.                                                       *
 *                                                                           *
 *  This Program is distributed in the hope that it will be useful,          *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the             *
 *  GNU General Public License for more details.                             *
 *                                                                           *
 *  You should have received a copy of the GNU General Public License        *
 *  along with GNU Make; see the file COPYING.  If not, write to             *
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.    *
 *  http://www.gnu.org/copyleft/gpl.html                                     *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)


interface

uses
  Windows, Messages, SysUtils, Variants, Classes,
  BDADriver, ActiveX, DCDVB_SDK_TuningPlugins;

const
  HDUS_XOR_UNKNOWN  = 0;
  HDUS_XOR_ON       = 1;
  HDUS_XOR_OFF      = 2;
  XORTABLE: array[0..183] of BYTE = (
$6D, $4E, $F0, $1E, $59, $8B, $B2, $F0, $31, $F9, $D0, $85, $DD, $84, $4A, $6C,
$15, $1C, $1C, $0A, $5B, $61, $6F, $1A, $AD, $60, $46, $F2, $D5, $03, $91, $C7,
$B7, $DE, $68, $9A, $07, $47, $62, $5E, $39, $D6, $7E, $45, $10, $9D, $1A, $C4,
$4C, $7F, $B7, $03, $CC, $AD, $72, $6C, $E9, $1B, $85, $A8, $EE, $4B, $A9, $3A,
$FF, $F1, $3B, $16, $E0, $59, $4C, $3A, $D5, $4A, $14, $95, $0A, $D4, $25, $23,
$F6, $2F, $E3, $57, $35, $CE, $52, $84, $11, $A3, $CB, $34, $A7, $15, $60, $E1,
$98, $71, $F4, $1F, $F7, $BD, $22, $4B, $61, $8A, $D6, $AF, $24, $27, $C3, $A7,
$07, $9B, $A5, $57, $83, $80, $58, $07, $1D, $DE, $0A, $D8, $B9, $0A, $BE, $AC,
$A9, $76, $00, $1C, $23, $9F, $82, $79, $41, $56, $C8, $83, $C3, $38, $37, $4B,
$5A, $ED, $C9, $D5, $F2, $87, $02, $C3, $79, $A0, $61, $D9, $63, $50, $CA, $72,
$C2, $51, $40, $98, $D9, $8B, $ED, $A9, $A1, $4C, $A5, $E7, $AF, $F2, $08, $4D,
$82, $F8, $D5, $02, $61, $D8, $C2, $E1
);

type
  TLogMethod   = procedure (Drv: TObject; AText:String) of Object;
  TTSMethod    = procedure (Drv: TObject; ABuffer: PByte; ASize: Integer) of Object;

  THDUS_Driver = class(TInterfacedObject,IDCDVBTuningPluginApplicationCallback)
  private
    { Private 錾 }
    FTuner     :TDCDVBTuningPlugin;
    FBDA       :TDCDVBTuningPluginDevice;
    FLogMethod :TLogMethod;
    FTSMethod  :TTSMethod;
    FFrequency :Integer;
    FBandWidth :Integer;
    FXORFlag   :Integer;
    function GetTunerCount: Integer;
    function GetTunerName(idx: Integer): String;
    function GetTunerID(idx: Integer): String;
  public
    { Public 錾 }
    constructor Create;
    destructor  Destroy; override;
    function Activate(ID:String): Boolean;
    function Tune(Freq:Integer; Band:Integer): Boolean;
    function Deactivate: Boolean;
    function ReceiveTSStream(ABuffer: PByte; ASize: Integer): Integer; stdcall;
    function LogEvent(AMethod: PChar; AText: PChar): Integer; stdcall;
    property LogMethod  :TLogMethod read FLogMethod  write FLogMethod;
    property TSMethod   :TTSMethod  read FTSMethod   write FTSMethod;
    property TunerCount :Integer    read GetTunerCount;
    property TunerName[idx: Integer] :String read GetTunerName;
    property TunerID  [idx: Integer] :String read GetTunerID;
    property Frequency  :Integer    read FFrequency;
    property BandWidth  :Integer    read FBandWidth;
    property XORFlag    :Integer    read FXORFlag    write FXORFlag;
  end;

implementation

// --------------------------------------------------------
//   vpeB֘A
// --------------------------------------------------------

function THDUS_Driver.GetTunerCount: Integer;
var
  cnt : Integer;
begin
  if Assigned(FTuner) then
  begin
    FTuner.get_TunerDevicesCount(cnt);
    Result := cnt;
  end
    else
  begin
    Result := -1;
  end;
end;

function THDUS_Driver.GetTunerName(idx: Integer): String;
var
  cnt : Integer;
  plg : IDCDVBTuningPluginDevice;
  prm : TDCDVBDeviceParams;
begin
  Result := '';
  if Assigned(FTuner) then
  begin
    FTuner.get_TunerDevicesCount(cnt);
    if (idx<cnt) then
    begin
      prm.Size := SizeOf(TDCDVBDeviceParams);
      FTuner.get_TunerDevice(idx,plg);
      plg.get_DeviceParams(prm);
      Result := prm.DeviceName;
    end;
  end;
end;

function THDUS_Driver.GetTunerID(idx: Integer): String;
var
  cnt : Integer;
  plg : IDCDVBTuningPluginDevice;
  prm : TDCDVBDeviceParams;
begin
  Result := '';
  if Assigned(FTuner) then
  begin
    FTuner.get_TunerDevicesCount(cnt);
    if (idx<cnt) then
    begin
      prm.Size := SizeOf(TDCDVBDeviceParams);
      FTuner.get_TunerDevice(idx,plg);
      plg.get_DeviceParams(prm);
      Result := prm.DeviceID;
    end;
  end;
end;

// --------------------------------------------------------
//   Constructor, Destructor
// --------------------------------------------------------

constructor THDUS_Driver.Create;
var
  GUID: TGUID;
begin
  FillChar(GUID,sizeof(GUID),0);
  FTuner     := TBDATuner.Create(GUID);
  FBDA       := nil;
  FLogMethod := nil;
  FTSMethod  := nil;
  FFrequency := 0;
  FBandWidth := 0;
  FXORFlag   := HDUS_XOR_UNKNOWN;
  inherited;
end;

destructor THDUS_Driver.Destroy;
begin
  if Assigned(FBDA)   then FreeAndNil(FBDA);
  if Assigned(FTuner) then FreeAndNil(FTuner);
  inherited;
end;


// --------------------------------------------------------
//   Activate
// --------------------------------------------------------

function THDUS_Driver.Activate(ID:String): Boolean;
var
  i,p :Integer;
  cnt :Integer;
  plg :IDCDVBTuningPluginDevice;
  prm :TDCDVBDeviceParams;
begin
  Result := False;
  p := -1;
  if Assigned(FTuner) then
  begin
    FTuner.get_TunerDevicesCount(cnt);
    for i:=0 to cnt-1 do
    begin
      prm.Size := SizeOf(TDCDVBDeviceParams);
      FTuner.get_TunerDevice(i,plg);
      plg.get_DeviceParams(prm);
      if (prm.DeviceID = ID) then
      begin
        p := i;
        break;
      end;
    end;
    //
    if (p>=0) then
    begin
      // create BDA Device
      FTuner.get_TunerDevice(p,plg);
      plg.get_DeviceParams(prm);
      FBDA := TBDADevice.Create(prm);
      FBDA.put_ApplicationCallback(self);
      // Activate
      FBDA.put_ControlMessage(DCDVB_TUNING_PLUGIN_CTRL_ACTIVATE,0,0);
      Result := True;
    end;
  end;
end;

// --------------------------------------------------------
//   Deactivate
// --------------------------------------------------------

function THDUS_Driver.Deactivate: Boolean;
begin
  Result := False;
  if Assigned(FBDA) then
  begin
    // Deactivate
    FBDA.put_ControlMessage(DCDVB_TUNING_PLUGIN_CTRL_DEACTIVATE,0,0);
    Result := True;
  end;
end;

// --------------------------------------------------------
//   Tune
// --------------------------------------------------------

function THDUS_Driver.Tune(Freq:Integer; Band:Integer): Boolean;
var
  tune :TDCDVBTTuneRequest;
begin
  Result := False;
  if Assigned(FBDA) then
  begin
    // Tune
    tune.Size         := SizeOf(TDCDVBTTuneRequest);
    tune.DeviceType   := DCDVB_TUNING_PLUGIN_DEVICE_TYPE_DVBT;
    tune.FrequencyKHz := Freq;
    tune.BandwidthMHz := Band;
    FBDA.put_ControlMessage(DCDVB_TUNING_PLUGIN_CTRL_TUNEREQUEST,Integer(@tune),0);
    FFrequency        := Freq;
    FBandWidth        := Band;
    Result := True;
  end;
end;


// --------------------------------------------------------
//   Interface
// --------------------------------------------------------

function THDUS_Driver.ReceiveTSStream(ABuffer: PByte; ASize: Integer): Integer;
var
  i   :Integer;
  siz :Integer;
  ptr :PByte;
  PID :WORD;
  tc  :Byte;
begin
  // XOR
  if (FXORFlag = HDUS_XOR_UNKNOWN) then
  begin
    ptr := ABuffer;
    i   := 0;
    while ((i<=(ASize-188)) and (ptr^ = $47) and (FXORFlag=HDUS_XOR_UNKNOWN)) do
    begin
      PID := ((PByte(Integer(ptr)+1)^ shl 8) or PByte(Integer(ptr)+2)^) and $1FFF;
      if (PID = $0000) then
      begin
        // PAT -> get TableʃR[h
        tc := PByte(Integer(ptr)+4)^;
        if (tc = $00) then
        begin
          // n[hEFA
          FXORFlag := HDUS_XOR_OFF;
        end
          else if (tc = $6D) then
        begin
          // n[hEFA
          FXORFlag := HDUS_XOR_ON;
        end;
      end;
      // Next packet
      inc(ptr,188);
      inc(i,188);
    end;
  end;

  // ςȂ珑o
  if (FXORFlag<>HDUS_XOR_UNKNOWN) then
  begin
    if Assigned(FTSMethod) then
    begin
      if (FXORFlag = HDUS_XOR_ON) then
      begin
        siz := ASize;
        ptr := ABuffer;
        while(siz>=188) do
        begin
          inc(ptr,4);
          for i:=0 to 183 do
          begin
            ptr^ := ptr^ xor XORTABLE[i];
            inc(ptr,1);
          end;
          siz := siz - 188;
        end;
      end;
      FTSMethod(Self,ABuffer,ASize);
    end;
  end;
  Result := S_OK;
end;

function THDUS_Driver.LogEvent(AMethod: PChar; AText: PChar): Integer;
begin
  if Assigned(FLogMethod) then
  begin
    FLogMethod(Self,'['+String(AMethod)+'] '+String(AText));
  end;
  Result := S_OK;
end;


end.
