library SegDriver_hdus;

uses
  SysUtils,
  Classes,
  Windows,
  HDUS_BDA in 'HDUS_BDA.pas';

{$R *.res}

const
  DLLNAME = 'HDUS w/YN_JP Driver';
  DLLDESC = 'MonsterTV HDUSYN_JP40___9 UNFyHWN3JgJ̉̂ΏۂƂhCoł';
  DLLVER  = $0100;
  DLLSEG  = 13;

var
  BaseInstance: THDUS_Driver;


function MakeDriverInstance:THDUS_Driver; stdcall;
begin
  Result := THDUS_Driver.Create;
end;

procedure FreeDriverInstance(HDUS:THDUS_Driver); stdcall;
begin
  if Assigned(HDUS) then HDUS.Free;
end;

function GetTunerCount:Integer; stdcall;
begin
  Result := BaseInstance.TunerCount;
end;

procedure GetTunerID(idx:Integer; pc:PChar; len:Integer); stdcall;
begin
  StrLCopy(pc,PChar(BaseInstance.TunerID[idx]),len);
end;

procedure GetTunerName(idx:Integer; pc:PChar; len:Integer); stdcall;
begin
  StrLCopy(pc,PChar(BaseInstance.TunerName[idx]),len);
end;

function Activate(HDUS:THDUS_Driver; ID:PChar): Boolean; stdcall;
begin
  if Assigned(HDUS) then
  begin
    Result := HDUS.Activate(String(ID));
  end
    else
  begin
    Result := False;
  end;
end;

function Tune(HDUS:THDUS_Driver; Freq:Integer): Boolean; stdcall;
begin
  if Assigned(HDUS) then
  begin
    Result := HDUS.Tune(Freq,7);
  end
    else
  begin
    Result := False;
  end;
end;

function Deactivate(HDUS:THDUS_Driver): Boolean; stdcall;
begin
  if Assigned(HDUS) then
  begin
    Result := HDUS.Deactivate;
  end
    else
  begin
    Result := False;
  end;
end;

procedure SetReceiveTSCallback(HDUS:THDUS_Driver; cb:TTSMethod); stdcall;
begin
  if Assigned(HDUS) then
  begin
    HDUS.TSMethod := cb;
  end;
end;

procedure SetLogCallback(HDUS:THDUS_Driver; cb:TLogMethod); stdcall;
begin
  if Assigned(HDUS) then
  begin
    HDUS.LogMethod := cb;
  end;
end;

function GetDLLName: PChar; stdcall;
begin
  Result := DLLNAME;
end;

function GetDLLDescriptor: PChar; stdcall;
begin
  Result := DLLDESC;
end;

function GetDLLVersion: DWORD; stdcall;
begin
  Result := DLLVER;
end;

function GetSegments: DWORD; stdcall;
begin
  Result := DLLSEG;
end;

exports
  MakeDriverInstance,
  FreeDriverInstance,
  GetTunerCount,
  GetTunerID,
  GetTunerName,
  Activate,
  Tune,
  Deactivate,
  SetReceiveTSCallback,
  SetLogCallback,
  GetDLLName,
  GetDLLDescriptor,
  GetDLLVersion,
  GetSegments;

procedure DLLEntry(ul_reason_for_call: DWORD);
begin
  case ul_reason_for_call of
    DLL_PROCESS_ATTACH:
      BaseInstance := THDUS_Driver.Create;
    DLL_PROCESS_DETACH:
      BaseInstance.Free;
    DLL_THREAD_ATTACH:
        ;
    DLL_THREAD_DETACH:
        ;
  end;
end;

begin
  DllProc := @DLLEntry;
  DLLEntry(DLL_PROCESS_ATTACH);
end.
