unit pDriver;

interface

uses
  Windows, SysUtils;

type
  TTSMethod    = procedure (Drv: TObject; ABuffer: PByte; ASize: Integer) of Object;
  TLogMethod   = procedure (Drv: TObject; AText: String) of Object;

  TMakeDriverInstance   = function :Pointer; stdcall;
  TFreeDriverInstance   = procedure (Drv:Pointer); stdcall;
  TGetTunerCount        = function :Integer; stdcall;
  TGetTunerID           = procedure (idx:Integer; pc:PChar; len:Integer); stdcall;
  TGetTunerName         = procedure (idx:Integer; pc:PChar; len:Integer); stdcall;
  TActivate             = function (Drv:Pointer; ID:PChar): Boolean; stdcall;
  TTune                 = function (Drv:Pointer; Freq:Integer): Boolean; stdcall;
  TDeactivate           = function (Drv:Pointer): Boolean; stdcall;
  TSetReceiveTSCallback = procedure (Drv:Pointer; cb:TTSMethod); stdcall;
  TSetLogCallback       = procedure (Drv:Pointer; cb:TLogMethod); stdcall;
  TGetDLLName           = function : PChar; stdcall;
  TGetDLLDescriptor     = function : PChar; stdcall;
  TGetDLLVersion        = function : DWORD; stdcall;
  TGetSegments          = function : DWORD; stdcall;

  TDriverDLL = class(TObject)
  private
    FDLLMakeDriverInstance   :TMakeDriverInstance;
    FDLLFreeDriverInstance   :TFreeDriverInstance;
    FDLLGetTunerCount        :TGetTunerCount;
    FDLLGetTunerID           :TGetTunerID;
    FDLLGetTunerName         :TGetTunerName;
    FDLLActivate             :TActivate;
    FDLLTune                 :TTune;
    FDLLDeactivate           :TDeactivate;
    FDLLSetReceiveTSCallback :TSetReceiveTSCallback;
    FDLLSetLogCallback       :TSetLogCallback;
    FDLLGetDLLName           :TGetDLLName;
    FDLLGetDLLDescriptor     :TGetDLLDescriptor;
    FDLLGetDLLVersion        :TGetDLLVersion;
    FDLLGetSegments          :TGetSegments;
    //
    FDLL                     :String;
    Handle                   :DWORD;
    function  GetName:String;
    function  GetDescriptor:String;
    function  GetDLLName: PChar;
    function  GetDLLDescriptor: PChar;
    function  GetDLLVersion: DWORD;
    function  GetSegments: DWORD;
    function  GetID(idx:Integer): String;
    function  GetTName(idx:Integer): String;
    function  GetTunerCount:Integer;
    procedure GetTunerID(idx:Integer; pc:PChar; len:Integer);
    procedure GetTunerName(idx:Integer; pc:PChar; len:Integer);
  public
    function  MakeDriverInstance:Pointer;
    procedure FreeDriverInstance(Drv:Pointer);
    function  Activate(Drv:Pointer; ID:PChar): Boolean;
    function  Tune(Drv:Pointer; Freq:Integer): Boolean;
    function  Deactivate(Drv:Pointer): Boolean;
    procedure SetReceiveTSCallback(Drv:Pointer; cb:TTSMethod);
    procedure SetLogCallback(Drv:Pointer; cb:TLogMethod);
    //
    constructor Create(DLLFile:String);
    destructor  Destroy; override;
    property    DLL:String                    read FDLL;
    property    Name:String                   read GetName;
    property    Descriptor:String             read GetDescriptor;
    property    Version:DWORD                 read GetDLLVersion;
    property    Segments:DWORD                read GetSegments;
    property    Count:Integer                 read GetTunerCount;
    property    TunerID[idx:Integer]:String   read GetID;
    property    TunerName[idx:Integer]:String read GetTName;
  end;


implementation

constructor TDriverDLL.Create(DLLFile:String);
begin
  FDLL := ExtractFileName(DLLFile);
  Handle := LoadLibrary(PChar(DLLFile));
  if (Handle<>0) then
  begin
    FDLLMakeDriverInstance   := GetProcAddress(Handle, 'MakeDriverInstance');
    FDLLFreeDriverInstance   := GetProcAddress(Handle, 'FreeDriverInstance');
    FDLLGetTunerCount        := GetProcAddress(Handle, 'GetTunerCount');
    FDLLGetTunerID           := GetProcAddress(Handle, 'GetTunerID');
    FDLLGetTunerName         := GetProcAddress(Handle, 'GetTunerName');
    FDLLActivate             := GetProcAddress(Handle, 'Activate');
    FDLLTune                 := GetProcAddress(Handle, 'Tune');
    FDLLDeactivate           := GetProcAddress(Handle, 'Deactivate');
    FDLLSetReceiveTSCallback := GetProcAddress(Handle, 'SetReceiveTSCallback');
    FDLLSetLogCallback       := GetProcAddress(Handle, 'SetLogCallback');
    FDLLGetDLLName           := GetProcAddress(Handle, 'GetDLLName');
    FDLLGetDLLDescriptor     := GetProcAddress(Handle, 'GetDLLDescriptor');
    FDLLGetDLLVersion        := GetProcAddress(Handle, 'GetDLLVersion');
    FDLLGetSegments          := GetProcAddress(Handle, 'GetSegments');
  end
    else
  begin
    FDLLMakeDriverInstance   := nil;
    FDLLFreeDriverInstance   := nil;
    FDLLGetTunerCount        := nil;
    FDLLGetTunerID           := nil;
    FDLLActivate             := nil;
    FDLLTune                 := nil;
    FDLLDeactivate           := nil;
    FDLLSetReceiveTSCallback := nil;
    FDLLGetDLLName           := nil;
    FDLLGetDLLDescriptor     := nil;
    FDLLGetDLLVersion        := nil;
    FDLLGetSegments          := nil;
  end;
end;

destructor TDriverDLL.Destroy;
begin
  if (Handle<>0) then
  begin
    FreeLibrary(Handle);
    Handle := 0;
  end;
  inherited;
end;

function TDriverDLL.GetName:String;
begin
  Result := String(GetDLLName);
end;

function TDriverDLL.GetDescriptor:String;
begin
  Result := String(GetDLLDescriptor);
end;

function TDriverDLL.GetID(idx:Integer): String;
const
  GETIDBUFSIZE = 65536;
var
  pc: PChar;
begin
  GetMem(pc,GETIDBUFSIZE);
  pc[0] := #0;
  GetTunerID(idx,pc,GETIDBUFSIZE);
  Result := String(pc);
  FreeMem(pc);
end;

function TDriverDLL.GetTName(idx:Integer): String;
const
  GETIDBUFSIZE = 65536;
var
  pc: PChar;
begin
  GetMem(pc,GETIDBUFSIZE);
  pc[0] := #0;
  GetTunerName(idx,pc,GETIDBUFSIZE);
  Result := String(pc);
  FreeMem(pc);
end;

function TDriverDLL.MakeDriverInstance:Pointer;
begin
  Result := nil;
  if Assigned(FDLLMakeDriverInstance) then
  begin
    Result := FDLLMakeDriverInstance;
  end;
end;

procedure TDriverDLL.FreeDriverInstance(Drv:Pointer);
begin
  if Assigned(FDLLFreeDriverInstance) then FDLLFreeDriverInstance(Drv);
end;

function TDriverDLL.GetTunerCount:Integer;
begin
  Result := 0;
  if Assigned(FDLLGetTunerCount) then Result := FDLLGetTunerCount;
end;

procedure TDriverDLL.GetTunerID(idx:Integer; pc:PChar; len:Integer);
begin
  if Assigned(FDLLGetTunerID) then FDLLGetTunerID(idx,pc,len);
end;

procedure TDriverDLL.GetTunerName(idx:Integer; pc:PChar; len:Integer);
begin
  if Assigned(FDLLGetTunerName) then FDLLGetTunerName(idx,pc,len);
end;

function TDriverDLL.Activate(Drv:Pointer; ID:PChar): Boolean;
begin
  Result := False;
  if Assigned(FDLLActivate) then Result := FDLLActivate(Drv,ID);
end;

function TDriverDLL.Tune(Drv:Pointer; Freq:Integer): Boolean;
begin
  Result := False;
  if Assigned(FDLLTune) then Result := FDLLTune(Drv,Freq);
end;

function TDriverDLL.Deactivate(Drv:Pointer): Boolean;
begin
  Result := False;
  if Assigned(FDLLDeactivate) then Result := FDLLDeactivate(Drv);
end;

procedure TDriverDLL.SetReceiveTSCallback(Drv:Pointer; cb:TTSMethod);
begin
  if Assigned(FDLLSetReceiveTSCallback) then FDLLSetReceiveTSCallback(Drv,cb);
end;

procedure TDriverDLL.SetLogCallback(Drv:Pointer; cb:TlogMethod);
begin
  if Assigned(FDLLSetLogCallback) then FDLLSetLogCallback(Drv,cb);
end;

function TDriverDLL.GetDLLName: PChar;
begin
  Result := nil;
  if Assigned(FDLLGetDLLName) then Result := FDLLGetDLLName;
end;

function TDriverDLL.GetDLLDescriptor: PChar;
begin
  Result := nil;
  if Assigned(FDLLGetDLLDescriptor) then Result := FDLLGetDLLDescriptor;
end;

function TDriverDLL.GetDLLVersion: DWORD;
begin
  Result := 0;
  if Assigned(FDLLGetDLLVersion) then Result := FDLLGetDLLVersion;
end;

function TDriverDLL.GetSegments: DWORD;
begin
  Result := 0;
  if Assigned(FDLLGetSegments) then Result := FDLLGetSegments;
end;


end.
