unit pEPGCollection;

interface

uses
  SysUtils, Classes, DateUtils, ActiveX, XMLDoc, XMLIntf, pLogUtils,
  Variants, Contnrs, Math, pProviderCollection,
  IdHTTP, IdDateTimeStamp;

const
  DATESEPARATEHOUR   =  5;

type
  TEPGItem = class(TObject)
    Title          :String;
    StartTime      :TDateTime;
    EndTime        :TDateTime;
    ProviderName   :String;
    ProviderRemote :Integer;
    ProviderCh     :Integer;
    Description    :String;
    Link           :String;
  end;

  TEPGItemBuffer = class(TObject)
    EPGList     :TObjectList;
    Date        :TDateTime;
    Area        :Integer;
    LastUpdate  :TDateTime;
  end;

  TEPGCollection = class(TObject)
    constructor Create(LogUtl:TLogUtil; Prov:TProviderCollection);
    destructor  Destroy; override;
  private
    { Private 錾 }
    IdHTTP    :TIdHTTP;
    Log       :TLogUtil;
    Provider  :TProviderCollection;
    Buffer    :TObjectList;
  public
    { Public 錾 }
    // obt@̊؂`FbNv
    function   DateTimeToDate(date: TDateTime):TDateTime;
    function   DatetoItemList(List:TObjectList; date: TDateTime; area:Integer):Boolean;
    function   RSStoItemList(List:TObjectList; URL:String): Boolean;
    procedure  SortByStartTime(List:TObjectList);
    procedure  SortByEndTime(List:TObjectList);
    procedure  SortByTitle(List:TObjectList);
    procedure  SortByProvider(List:TObjectList);
  end;

implementation

constructor TEPGCollection.Create(LogUtl:TLogUtil; Prov:TProviderCollection);
begin
  CoInitialize(nil);
  Log      := LogUtl;
  Provider := Prov;
  IdHTTP   := TIdHTTP.Create(nil);
  Buffer   := TObjectList.Create;
end;

destructor TEPGCollection.Destroy;
begin
  inherited;
  if Assigned(IdHTTP) then FreeAndNil(IdHTTP);
  if Assigned(Buffer) then FreeAndNil(Buffer);
  CoUninitialize;
end;

function TEPGCollection.DateTimeToDate(date: TDateTime):TDateTime;
var
  date_s: TDateTime;
begin
  date_s := DateOf(date) + EncodeTime(DATESEPARATEHOUR,0,0,0);
  if (CompareDateTime(date_s,date)<=0) then
  begin
    Result := DateOf(date) + EncodeTime(DATESEPARATEHOUR,0,0,0);
  end
    else
  begin
    Result := IncDay(DateOf(date),-1) + EncodeTime(DATESEPARATEHOUR,0,0,0);
  end;
end;

function TEPGCollection.RSStoItemList(List:TObjectList; URL: String): Boolean;
var
  xml          :String;
  XMLd         :IXMLDocument;
  XMLn         :IXMLNode;
  Item         :TEPGItem;
  Title        :String;
  Description  :String;
  Link         :String;
  pubDate      :String;
  Category     :String;
  val          :OleVariant;
  IdDateTime   :TIdDateTimeStamp;
  StartTime    :TDateTime;
  EndTime      :TDateTime;
  TitleSect    :array[0..2] of String;
  i,j,p        :Integer;
begin
  IdDateTime := TIdDateTimeStamp.Create(nil);
  Result     := True;
  try
    xml  := IdHTTP.Get(URL);
    XMLd := LoadXMLData(xml);
    XMLn := XMLd.DocumentElement;
    for i:=0 to XMLn.ChildNodes.Count-1 do
    begin
      if (LowerCase(XMLn.ChildNodes[i].NodeName)='item') then
      begin
        Title := ''; val := XMLn.ChildNodes[i].ChildValues['title'];
        if (val <> Null) then Title := val;
        Description := ''; val := XMLn.ChildNodes[i].ChildValues['description'];
        if (val <> Null) then Description := val;
        Link := ''; val := XMLn.ChildNodes[i].ChildValues['link'];
        if (val <> Null) then Link := val;
        pubDate := ''; val := XMLn.ChildNodes[i].ChildValues['pubDate'];
        if (val <> Null) then pubDate := val;
        Category := ''; val := XMLn.ChildNodes[i].ChildValues['Category'];
        if (val <> Null) then Category := val;
        // ---------- 
        // Jn
        IdDateTime.SetFromRFC822(pubDate);
        StartTime := IdDateTime.GetAsTDateTime;
        // ^Cg
        for j:=0 to 2 do
        begin
          p := Pos(' ',Title);
          if (p=0) then break;
          TitleSect[j] := Copy(Title,1,p-1);
          Title := Copy(Title,p+1,Length(Title));
        end;
        EndTime := StartTime;
        p := Pos('`',TitleSect[1]);
        if (p>0) then
        begin
          TitleSect[1] := Copy(TitleSect[1],p+2,Length(TitleSect[1]));
          p := Pos(':',TitleSect[1]);
          if (p>0) then
          begin
            EndTime := DateOf(EndTime)
                      + EncodeTime(
                         StrToIntDef(Copy(TitleSect[1],1,p-1),0),
                         StrToIntDef(Copy(TitleSect[1],p+1,Length(TitleSect[1])),0),0,0);
          end;
          if (EndTime<StartTime) then EndTime := IncDay(EndTime);
        end;

        // Item
        Item := TEPGItem.Create;
        Item.StartTime      := StartTime;
        Item.EndTime        := EndTime;
        Item.Title          := Title;
        Item.ProviderName   := TitleSect[2];
        Item.ProviderRemote := Provider.NameToRemote(TitleSect[2]);
        Item.ProviderCh     := Provider.NameToCh(TitleSect[2]);;
        Item.Description    := Description;
        Item.Link           := Link;
        List.Add(Item);
      end;
    end;
  except
    Result := False;
  end;
  IdDateTime.Free;
end;

function TEPGCollection.DatetoItemList(List:TObjectList; date: TDateTime; area:Integer):Boolean;
var
  i: Integer;
  ItemBuffer: TEPGItemBuffer;
  AreaCode,ymd :String;
  d1,d2: TDateTime;
  NeedUpdate: Boolean;
  BufIndex: Integer;
begin
  List.Clear;
  date := DateTimeToDate(date);
  NeedUpdate := True;
  BufIndex   := -1;
  // obt@
  for i:=0 to Buffer.Count-1 do
  begin
    ItemBuffer := TEPGItemBuffer(Buffer.Items[i]);
    if ((CompareDate(date,DateTimeToDate(ItemBuffer.Date))=0) and (ItemBuffer.Area = area)) then
    begin
      // LastUpdatemF
      BufIndex := i;
      d1 := DateTimeToDate(ItemBuffer.LastUpdate);
      d2 := DateTimeToDate(Now);
      if (CompareDate(d1,d2)>=0) then
      begin
        // LastUpdateAAf[^ꍇ̓obt@gp
        NeedUpdate := False;
      end;
    end;
  end;
  //
  if (NeedUpdate) then
  begin
    AreaCode := Format('%3.3d',[area]);
    ymd      := Format('%4.4d%2.2d%2.2d',[YearOf(date),MonthOf(date),DayOf(date)]);
    Result   := RSStoItemList(List,'http://tv.nikkansports.com/tv.php?mode=04&site=007&lhour=24&shour='+IntToStr(DATESEPARATEHOUR)+'&sdate='+ymd+'&category=g&template=rss&area='+AreaCode+'&pageCharSet=UTF8');
    if (BufIndex<0) then
    begin
      ItemBuffer := TEPGItemBuffer.Create;
      ItemBuffer.Date       := date;
      ItemBuffer.Area       := area;
      Buffer.Add(ItemBuffer);
    end
      else
    begin
      ItemBuffer := TEPGItemBuffer(Buffer.Items[BufIndex]);
      ItemBuffer.EPGList.Free;
    end;
    ItemBuffer.EPGList    := TObjectList.Create;
    ItemBuffer.EPGList.Assign(List);
    List.OwnsObjects := False;        // BufferOwn
    ItemBuffer.LastUpdate := Now;
  end
    else
  begin
    List.Assign(TEPGItemBuffer(Buffer.Items[BufIndex]).EPGList);
    List.OwnsObjects := False;        // BufferOwn
    Result := True;
  end;
end;

//
function ByStartTime(Item1, Item2: Pointer): Integer;
begin
  Result := CompareDateTime(TEPGItem(Item1).StartTime, TEPGItem(Item2).StartTime);
end;

function ByEndTime(Item1, Item2: Pointer): Integer;
begin
  Result := CompareDateTime(TEPGItem(Item1).EndTime, TEPGItem(Item2).EndTime);
end;

function ByTitle(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TEPGItem(Item1).Title, TEPGItem(Item2).Title);
end;

function ByProvider(Item1, Item2: Pointer): Integer;
begin
  Result := CompareValue(TEPGItem(Item1).ProviderRemote, TEPGItem(Item2).ProviderRemote);
end;

procedure TEPGCollection.SortByStartTime(List:TObjectList);
begin
  List.Sort(@ByStartTime);
end;

procedure TEPGCollection.SortByEndTime(List:TObjectList);
begin
  List.Sort(@ByEndTime);
end;

procedure TEPGCollection.SortByTitle(List:TObjectList);
begin
  List.Sort(@ByTitle);
end;

procedure TEPGCollection.SortByProvider(List:TObjectList);
begin
  List.Sort(@ByProvider);
end;


end.
