unit pMainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, pDriver, B25D_delphi, IdBaseComponent, IdComponent, IdUDPBase,
  IdUDPServer, IdUDPClient, StdCtrls, ExtCtrls, INIFiles, ShellAPI, Contnrs,
  pProviderCollection, pEPGCollection, pLogUtils;

const
  WM_APP_REMOTEKEY  = $8009;

type
  TControlForm = class(TForm)
    IdUDPClient1: TIdUDPClient;
    Panel1: TPanel;
    ChannelNumberLabel: TLabel;
    Panel2: TPanel;
    Timer1: TTimer;
    ProviderNameLabel: TLabel;
    EPGNameLabel: TLabel;
    EPGMemo: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure EPGMemoDblClick(Sender: TObject);
  private
    { Private 錾 }
    Log             : TLogUtil;
    FEPGCollection  : TEPGCollection;
    FProvCollection : TProviderCollection;
    HDUS            : Pointer;
    DriverPath      : String;
    B25Dec          : TB25Decoder;
    TunerState      : Boolean;
    TunerCh         : Integer;
    LinkURL         : String;
    //
    FTunerID    :String;
    FUDPAddr    :String;
    FUDPPort    :Integer;
    FAreaCode   :Integer;
    FExecPlayer :String;
    procedure ReceiveTSStream(Drv:TObject; ABuffer: PByte; ASize: Integer);
    procedure LogEvent(Drv:TObject; AText: String);
    procedure ReadSetting;
    procedure WriteSetting;
    procedure TunerON;
    procedure TunerOFF;
  public
    { Public 錾 }
    procedure GetRemote(var Msg:TMessage);message WM_APP_REMOTEKEY;
    property  TunerID:String      read FTunerID    write FTunerID;
    property  UDPAddr:String      read FUDPAddr    write FUDPAddr;
    property  UDPPort:Integer     read FUDPPort    write FUDPPort;
    property  AreaCode:Integer    read FAreaCode   write FAreaCode;
    property  ExecPlayer:String   read FExecPlayer write FExecPlayer;
  end;

var
  ControlForm: TControlForm;
  HDUSDLL    : TDriverDLL;

procedure SetRemoteMessageWnd(Wnd:THandle); stdcall; external 'HDUS_KeyHookDLL.dll';
procedure SetKeyboardHook; stdcall; external 'HDUS_KeyHookDLL.dll';
procedure ResetKeyboardHook; stdcall; external 'HDUS_KeyHookDLL.dll';

implementation

{$R *.dfm}

procedure TControlForm.ReadSetting;
var
  INI: TINIFile;
begin
  INI := TINIFile.Create(ChangeFileExt(Application.ExeName,'.ini'));
  try
    FTunerID    := INI.ReadString ('Setting','TunerID'    ,'');
    FUDPAddr    := INI.ReadString ('Setting','Address'    ,'127.0.0.1');
    FUDPPort    := INI.ReadInteger('Setting','SendPort'   ,1234);
    FAreaCode   := INI.ReadInteger('Setting','AreaCode'   ,8);
    FExecPlayer := INI.ReadString ('Setting','ExecPlayer' ,'');
  except
  end;
  INI.Free;
end;

procedure TControlForm.WriteSetting;
var
  INI: TINIFile;
begin
  INI := TINIFile.Create(ChangeFileExt(Application.ExeName,'.ini'));
  try
//  INI.WriteString ('Setting','TunerID'    ,FTunerID    );
    INI.WriteString ('Setting','Address'    ,FUDPAddr    );
    INI.WriteInteger('Setting','SendPort'   ,FUDPPort    );
    INI.WriteInteger('Setting','AreaCode'   ,FAreaCode   );
//  INI.WriteString ('Setting','ExecPlayer' ,FExecPlayer );
  except
  end;
  INI.Free;
end;

procedure TControlForm.TunerON;
var
  DefaultTuner: String;
begin
  DefaultTuner := '';
  if (TunerID = '') then
  begin
    if (HDUSDLL.Count>0) then DefaultTuner := HDUSDLL.TunerID[0];
  end
    else
  begin
    DefaultTuner := TunerID;
  end;
  B25Dec.Reset;
  if HDUSDLL.Activate(HDUS,PChar(DefaultTuner)) then
  begin
    HDUSDLL.Tune(HDUS,TunerCh*1000);
    TunerState := True;
  end;
end;

procedure TControlForm.TunerOFF;
begin
  if Assigned(HDUS) then HDUSDLL.Deactivate(HDUS);
  TunerState := False;
end;

procedure TControlForm.FormCreate(Sender: TObject);
begin
  DriverPath := IncludeTrailingPathDelimiter(ExtractFilePath(Application.ExeName));
  ReadSetting;
  //
  Log             := TLogUtil.Create;
  Log.LogCallback := nil;
  FProvCollection := TProviderCollection.Create(Log,ChangeFileExt(Application.ExeName,'.ch'));
  FEPGCollection  := TEPGCollection.Create(Log,FProvCollection);
  TunerCh         := FProvCollection.NameToCh(FProvCollection.NameList[0]);
  TunerState      := False;
  //
  B25Dec  := TB25Decoder.Create;
  B25Dec.Initialize(4);
  HDUSDLL := TDriverDLL.Create(DriverPath+'SegDriver_hdus.dll');
  HDUS    := HDUSDLL.MakeDriverInstance;
  HDUSDLL.SetReceiveTSCallback(HDUS,ReceiveTSStream);
  HDUSDLL.SetLogCallback(HDUS,LogEvent);
  //
  SetRemoteMessageWnd(Handle);
  SetKeyboardHook;
  //
  TunerON;
end;

procedure TControlForm.FormDestroy(Sender: TObject);
begin
  WriteSetting;
  if Assigned(HDUS)    then HDUSDLL.Deactivate(HDUS);
  if Assigned(HDUSDLL) then HDUSDLL.Free;
  if Assigned(B25Dec)  then B25Dec.Free;
  ResetKeyboardHook;
end;

procedure TControlForm.GetRemote(var Msg:TMessage);
var
  Key: WORD;
  Rem,Ch: Integer;
begin
  Key := WORD(Msg.WParam);
  if (Key = 2) then
  begin
    // d{^
    if (TunerState) then TunerOFF else TunerON;
  end
    else if ((Key >= 4) and (Key <= 15)) then
  begin
    // `l{^
    if (TunerState) then
    begin
      Rem := Key-3;
      Ch := FProvCollection.RemoteToCh(Rem);
      if (Ch>0) then
      begin
        TunerCh := ch;
        IdUDPClient1.Active := False;
        HDUSDLL.Tune(HDUS,TunerCh*1000);
        IdUDPClient1.Active := True;
      end;
    end;
  end
    else if (Key = 21) then
  begin
    // j[
    if (FExecPlayer<>'') then
    begin
      WinExec(PChar(FExecPlayer),SW_NORMAL);
    end;
  end;
end;

procedure TControlForm.ReceiveTSStream(Drv:TObject; ABuffer: PByte; ASize: Integer);
var
  DecBuffer :PByte;
  DecSize   :DWORD;
begin
  if Assigned(B25Dec) then
  begin
    if (B25Dec.DLLReady) and (B25Dec.Initialized) then
    begin
      B25Dec.Decode(ABuffer,ASize,DecBuffer,DecSize);
      IdUDPClient1.Active := True;
      IdUDPClient1.SendBuffer(UDPAddr,UDPPort,DecBuffer^,DecSize);
    end;
  end;
end;

procedure TControlForm.LogEvent(Drv:TObject; AText: String);
begin
  //
end;

procedure TControlForm.Timer1Timer(Sender: TObject);
var
  str: String;
  EPGList: TObjectList;
  EPGItem: TEPGItem;
  i: Integer;
begin
  if (not TunerState) then
  begin
    if (ChannelNumberLabel.Caption<>'') then ChannelNumberLabel.Caption := '';
    if (ProviderNameLabel.Caption <>'') then ProviderNameLabel.Caption := '';
    if (EPGNameLabel.Caption      <>'') then
    begin
      EPGNameLabel.Caption := '';
      EPGMemo.Clear;
      LinkURL := '';
    end;
  end
    else
  begin
    str := IntToStr(FProvCollection.ChToRemote(TunerCh));
    if (ChannelNumberLabel.Caption<>str) then ChannelNumberLabel.Caption := str;
    str := FProvCollection.ChToName(TunerCh);
    if (ProviderNameLabel.Caption<>str)  then ProviderNameLabel.Caption := str;
    EPGList := TObjectList.Create;
    FEPGCollection.DatetoItemList(EPGList,Now,AreaCode);
    EPGItem := nil;
    for i:=0 to EPGList.Count-1 do
    begin
      EPGItem := TEPGItem(EPGList.Items[i]);
      if (((TunerCh<0) or (TunerCh=EPGItem.ProviderCh)) and (EPGItem.StartTime<=Now) and (EPGItem.EndTime>=Now)) then
      begin
        break;
      end;
      EPGItem := nil;
    end;
    if Assigned(EPGItem) then
    begin
      str := EPGItem.Title;
      if (EPGNameLabel.Caption<>str) then
      begin
        EPGNameLabel.Caption := str;
        EPGMemo.Clear;
        EPGMemo.Text := EPGItem.Description;
        LinkURL := EPGItem.Link;
      end;
    end;
  end;
end;

procedure TControlForm.EPGMemoDblClick(Sender: TObject);
begin
  ShellExecute(Handle,'open',PChar(LinkURL),nil,nil,SW_NORMAL);
end;

end.
